@@ -200,6 +200,8 @@ class curry(object):
 
         self.__doc__ = getattr(func, '__doc__', None)
         self.__name__ = getattr(func, '__name__', '<curry>')
+        self.__module__ = getattr(func, '__module__', None)
+        self.__qualname__ = getattr(func, '__qualname__', None)
         self._sigspec = None
         self._has_unknown_args = None
 
@@ -324,27 +326,43 @@ class curry(object):
     def __reduce__(self):
         func = self.func
         modname = getattr(func, '__module__', None)
-        funcname = getattr(func, '__name__', None)
-        if modname and funcname:
-            module = import_module(modname)
-            obj = getattr(module, funcname, None)
-            if obj is self:
-                return funcname
-            elif isinstance(obj, curry) and obj.func is func:
-                func = '%s.%s' % (modname, funcname)
+        qualname = getattr(func, '__qualname__', None)
+        if qualname is None:  # pragma: py3 no cover
+            qualname = getattr(func, '__name__', None)
+        is_decorated = None
+        if modname and qualname:
+            attrs = []
+            obj = import_module(modname)
+            for attr in qualname.split('.'):
+                if isinstance(obj, curry):  # pragma: py2 no cover
+                    attrs.append('func')
+                    obj = obj.func
+                obj = getattr(obj, attr, None)
+                if obj is None:
+                    break
+                attrs.append(attr)
+            if isinstance(obj, curry) and obj.func is func:
+                is_decorated = obj is self
+                qualname = '.'.join(attrs)
+                func = '%s:%s' % (modname, qualname)
 
         # functools.partial objects can't be pickled
         userdict = tuple((k, v) for k, v in self.__dict__.items()
-                         if k != '_partial')
-        state = (type(self), func, self.args, self.keywords, userdict)
+                         if k not in ('_partial', '_sigspec'))
+        state = (type(self), func, self.args, self.keywords, userdict,
+                 is_decorated)
         return (_restore_curry, state)
 
 
-def _restore_curry(cls, func, args, kwargs, userdict):
+def _restore_curry(cls, func, args, kwargs, userdict, is_decorated):
     if isinstance(func, str):
-        modname, funcname = func.rsplit('.', 1)
-        module = import_module(modname)
-        func = getattr(module, funcname).func
+        modname, qualname = func.rsplit(':', 1)
+        obj = import_module(modname)
+        for attr in qualname.split('.'):
+            obj = getattr(obj, attr)
+        if is_decorated:
+            return obj
+        func = obj.func
     obj = cls(func, *args, **(kwargs or {}))
     obj.__dict__.update(userdict)
     return obj
