@@ -1,12 +1,12 @@
 import itertools
-from .itertoolz import frequencies
+from .itertoolz import frequencies, pluck, getter
 from .compatibility import map
 
 
 __all__ = ('countby', 'partitionby')
 
 
-def countby(func, seq):
+def countby(key, seq):
     """ Count elements of a collection by a key function
 
     >>> countby(len, ['cat', 'mouse', 'dog'])
@@ -19,7 +19,9 @@ def countby(func, seq):
     See Also:
         groupby
     """
-    return frequencies(map(func, seq))
+    if not callable(key):
+        key = getter(key)
+    return frequencies(map(key, seq))
 
 
 def partitionby(func, seq):
@@ -42,6 +44,4 @@ def partitionby(func, seq):
         groupby
         itertools.groupby
     """
-    # Note: applying `list` seems to be required both Python 2 and 3
-    # compatible (Python 3 works without it).
-    return (tuple(v) for k, v in itertools.groupby(seq, key=func))
+    return map(tuple, pluck(1, itertools.groupby(seq, key=func)))
